@@ -1,6 +1,8 @@
 # Authors: 
 #   Trevor Perrin
 #   Google - handling CertificateRequest.certificate_types
+#   Google (adapted by Sam Rushing) - NPN support
+#   Dimitris Moraitis - Anon ciphersuites
 #
 # See the LICENSE file for legal information regarding use of this file.
 
@@ -108,10 +110,12 @@ class ClientHello(HandshakeMsg):
         self.compression_methods = []   # a list of 8-bit values
         self.srp_username = None        # a string
         self.tack = False
+        self.supports_npn = False
+        self.server_name = ""
 
     def create(self, version, random, session_id, cipher_suites,
                certificate_types=None, srp_username=None,
-               tack=False):
+               tack=False, supports_npn=False, server_name=""):
         self.client_version = version
         self.random = random
         self.session_id = session_id
@@ -120,6 +124,8 @@ class ClientHello(HandshakeMsg):
         self.compression_methods = [0]
         self.srp_username = srp_username
         self.tack = tack
+        self.supports_npn = supports_npn
+        self.server_name = server_name
         return self
 
     def parse(self, p):
@@ -157,8 +163,22 @@ class ClientHello(HandshakeMsg):
                         self.certificate_types = p.getVarList(1, 1)
                     elif extType == ExtensionType.tack:
                         self.tack = True
+                    elif extType == ExtensionType.supports_npn:
+                        self.supports_npn = True
+                    elif extType == ExtensionType.server_name:
+                        serverNameListBytes = p.getFixBytes(extLength)
+                        p2 = Parser(serverNameListBytes)
+                        p2.startLengthCheck(2)
+                        while 1:
+                            if p2.atLengthCheck():
+                                break # no host_name, oh well
+                            name_type = p2.get(1)
+                            hostNameBytes = p2.getVarBytes(2)
+                            if name_type == NameType.host_name:
+                                self.server_name = bytesToString(hostNameBytes)
+                                break
                     else:
-                        p.getFixBytes(extLength)
+                        _ = p.getFixBytes(extLength)
                     index2 = p.index
                     if index2 - index1 != extLength:
                         raise SyntaxError("Bad length for extension_data")
@@ -185,6 +205,12 @@ class ClientHello(HandshakeMsg):
             w2.add(ExtensionType.srp, 2)
             w2.add(len(self.srp_username)+1, 2)
             w2.addVarSeq(stringToBytes(self.srp_username), 1, 1)
+        if self.server_name:
+            w2.add(ExtensionType.server_name, 2)
+            w2.add(len(self.server_name)+5, 2)
+            w2.add(len(self.server_name)+3, 2)            
+            w2.add(NameType.host_name, 1)
+            w2.addVarSeq(stringToBytes(self.server_name), 1, 2) 
         if self.tack:
             w2.add(ExtensionType.tack, 2)
             w2.add(0, 2)
@@ -193,6 +219,13 @@ class ClientHello(HandshakeMsg):
             w.bytes += w2.bytes
         return self.postWrite(w)
 
+class BadNextProtos(Exception):
+    def __init__(self, l):
+        self.length = l
+
+    def __str__(self):
+        return 'Cannot encode a list of next protocols because it contains an element with invalid length %d. Element lengths must be 0 < x < 256' % self.length
+
 class ServerHello(HandshakeMsg):
     def __init__(self):
         HandshakeMsg.__init__(self, HandshakeType.server_hello)
@@ -202,10 +235,11 @@ class ServerHello(HandshakeMsg):
         self.cipher_suite = 0
         self.certificate_type = CertificateType.x509
         self.compression_method = 0
-        self.tackExt = None 
+        self.tackExt = None
+        self.next_protos_advertised = None
 
     def create(self, version, random, session_id, cipher_suite,
-               certificate_type, tackExt):
+               certificate_type, tackExt, next_protos_advertised):
         self.server_version = version
         self.random = random
         self.session_id = session_id
@@ -213,6 +247,7 @@ class ServerHello(HandshakeMsg):
         self.certificate_type = certificate_type
         self.compression_method = 0
         self.tackExt = tackExt
+        self.next_protos_advertised = next_protos_advertised
         return self
 
     def parse(self, p):
@@ -233,14 +268,23 @@ class ServerHello(HandshakeMsg):
                         raise SyntaxError()
                     self.certificate_type = p.get(1)
                 elif extType == ExtensionType.tack and tackpyLoaded:
-                    self.tackExt = TACK_Extension()
-                    self.tackExt.parse(p.getFixBytes(extLength))
+                    self.tackExt = TackExtension(p.getFixBytes(extLength))
                 else:
                     p.getFixBytes(extLength)
                 soFar += 4 + extLength
         p.stopLengthCheck()
         return self
 
+    def __next_protos_encoded(self):
+        a = []
+        for e in self.next_protos_advertised:
+            if len(e) > 255 or len(e) == 0:
+                raise BadNextProtos(len(e))
+            a.append(chr(len(e)))
+            a.append(e)
+
+        return [ord(x) for x in ''.join(a)]
+
     def write(self):
         w = Writer()
         w.add(self.server_version[0], 1)
@@ -257,10 +301,15 @@ class ServerHello(HandshakeMsg):
             w2.add(1, 2)
             w2.add(self.certificate_type, 1)
         if self.tackExt:
-            b = self.tackExt.write()
+            b = self.tackExt.serialize()
             w2.add(ExtensionType.tack, 2)
             w2.add(len(b), 2)
             w2.bytes += b
+        if self.next_protos_advertised is not None:
+            encoded_next_protos_advertised = self.__next_protos_encoded()
+            w2.add(ExtensionType.supports_npn, 2)
+            w2.add(len(encoded_next_protos_advertised), 2)
+            w2.addFixSeq(encoded_next_protos_advertised, 1)
         if len(w2.bytes):
             w.add(len(w2.bytes), 2)
             w.bytes += w2.bytes        
@@ -365,6 +414,10 @@ class ServerKeyExchange(HandshakeMsg):
         self.srp_g = 0L
         self.srp_s = createByteArraySequence([])
         self.srp_B = 0L
+        # Anon DH params:
+        self.dh_p = 0L
+        self.dh_g = 0L
+        self.dh_Ys = 0L
         self.signature = createByteArraySequence([])
 
     def createSRP(self, srp_N, srp_g, srp_s, srp_B):
@@ -373,26 +426,44 @@ class ServerKeyExchange(HandshakeMsg):
         self.srp_s = srp_s
         self.srp_B = srp_B
         return self
+    
+    def createDH(self, dh_p, dh_g, dh_Ys):
+        self.dh_p = dh_p
+        self.dh_g = dh_g
+        self.dh_Ys = dh_Ys
+        return self
 
     def parse(self, p):
         p.startLengthCheck(3)
-        self.srp_N = bytesToNumber(p.getVarBytes(2))
-        self.srp_g = bytesToNumber(p.getVarBytes(2))
-        self.srp_s = p.getVarBytes(1)
-        self.srp_B = bytesToNumber(p.getVarBytes(2))
-        if self.cipherSuite in CipherSuite.srpCertSuites:
-            self.signature = p.getVarBytes(2)
+        if self.cipherSuite in CipherSuite.srpAllSuites:
+            self.srp_N = bytesToNumber(p.getVarBytes(2))
+            self.srp_g = bytesToNumber(p.getVarBytes(2))
+            self.srp_s = p.getVarBytes(1)
+            self.srp_B = bytesToNumber(p.getVarBytes(2))
+            if self.cipherSuite in CipherSuite.srpCertSuites:
+                self.signature = p.getVarBytes(2)
+        elif self.cipherSuite in CipherSuite.anonSuites:
+            self.dh_p = bytesToNumber(p.getVarBytes(2))
+            self.dh_g = bytesToNumber(p.getVarBytes(2))
+            self.dh_Ys = bytesToNumber(p.getVarBytes(2))
         p.stopLengthCheck()
         return self
 
     def write(self):
         w = Writer()
-        w.addVarSeq(numberToBytes(self.srp_N), 1, 2)
-        w.addVarSeq(numberToBytes(self.srp_g), 1, 2)
-        w.addVarSeq(self.srp_s, 1, 1)
-        w.addVarSeq(numberToBytes(self.srp_B), 1, 2)
-        if self.cipherSuite in CipherSuite.srpCertSuites:
-            w.addVarSeq(self.signature, 1, 2)
+        if self.cipherSuite in CipherSuite.srpAllSuites:
+            w.addVarSeq(numberToBytes(self.srp_N), 1, 2)
+            w.addVarSeq(numberToBytes(self.srp_g), 1, 2)
+            w.addVarSeq(self.srp_s, 1, 1)
+            w.addVarSeq(numberToBytes(self.srp_B), 1, 2)
+            if self.cipherSuite in CipherSuite.srpCertSuites:
+                w.addVarSeq(self.signature, 1, 2)
+        elif self.cipherSuite in CipherSuite.anonSuites:
+            w.addVarSeq(numberToBytes(self.dh_p), 1, 2)
+            w.addVarSeq(numberToBytes(self.dh_g), 1, 2)
+            w.addVarSeq(numberToBytes(self.dh_Ys), 1, 2)
+            if self.cipherSuite in []: # TODO support for signed_params
+                w.addVarSeq(self.signature, 1, 2)
         return self.postWrite(w)
 
     def hash(self, clientRandom, serverRandom):
@@ -436,7 +507,11 @@ class ClientKeyExchange(HandshakeMsg):
     def createRSA(self, encryptedPreMasterSecret):
         self.encryptedPreMasterSecret = encryptedPreMasterSecret
         return self
-
+    
+    def createDH(self, dh_Yc):
+        self.dh_Yc = dh_Yc
+        return self
+    
     def parse(self, p):
         p.startLengthCheck(3)
         if self.cipherSuite in CipherSuite.srpAllSuites:
@@ -449,6 +524,8 @@ class ClientKeyExchange(HandshakeMsg):
                     p.getFixBytes(len(p.bytes)-p.index)
             else:
                 raise AssertionError()
+        elif self.cipherSuite in CipherSuite.anonSuites:
+            self.dh_Yc = bytesToNumber(p.getVarBytes(2))            
         else:
             raise AssertionError()
         p.stopLengthCheck()
@@ -465,6 +542,8 @@ class ClientKeyExchange(HandshakeMsg):
                 w.addFixSeq(self.encryptedPreMasterSecret, 1)
             else:
                 raise AssertionError()
+        elif self.cipherSuite in CipherSuite.anonSuites:
+            w.addVarSeq(numberToBytes(self.dh_Yc), 1, 2)            
         else:
             raise AssertionError()
         return self.postWrite(w)
@@ -510,6 +589,27 @@ class ChangeCipherSpec:
         return w.bytes
 
 
+class NextProtocol(HandshakeMsg):
+    def __init__(self):
+        self.contentType = ContentType.handshake
+        self.next_proto = None
+
+    def create(self, next_proto):
+        self.next_proto = next_proto
+
+    def parse(self, p):
+        p.startLengthCheck(3)
+        self.next_proto = p.getVarBytes(1)
+        _ = p.getVarBytes(1)
+        p.stopLengthCheck()
+        return self
+
+    def write(self):
+        w = Writer()
+        w.addVarSeq(self.next_proto, 1, 1)
+        w.addVarSeq('\x00' * 32, 1, 32 - ((len(self.next_proto) + 2) % 32));
+        return self.postWrite(w)
+
 class Finished(HandshakeMsg):
     def __init__(self, version):
         HandshakeMsg.__init__(self, HandshakeType.finished)
@@ -555,4 +655,4 @@ class ApplicationData:
         return self
 
     def write(self):
-        return self.bytes
\ No newline at end of file
+        return self.bytes
